@@ -1,3 +1,4 @@
+# Copyright 2014 Red Hat
 # Copyright 2013 Mirantis.inc
 # All Rights Reserved.
 #
@@ -16,6 +17,7 @@
 """Provision test environment for specific DB backends"""
 
 import abc
+import debtcollector
 import logging
 import os
 import random
@@ -31,7 +33,7 @@ import testresources
 
 from oslo_db._i18n import _LI
 from oslo_db import exception
-from oslo_db.sqlalchemy.compat import utils as compat_utils
+from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import session
 from oslo_db.sqlalchemy import utils
 
@@ -39,52 +41,127 @@ LOG = logging.getLogger(__name__)
 
 
 class ProvisionedDatabase(object):
-    pass
+    """Represents a database engine pointing to a DB ready to run tests.
+
+    backend: an instance of :class:`.Backend`
+
+    enginefacade: an instance of :class:`._TransactionFactory`
+
+    engine: a SQLAlchemy :class:`.Engine`
+
+    db_token: if provision_new_database were used, this is the randomly
+              generated name of the database.  Note that with SQLite memory
+              connections, this token is ignored.   For a database that
+              wasn't actually created, will be None.
+
+    """
+
+    __slots__ = 'backend', 'enginefacade', 'engine', 'db_token'
+
+    def __init__(self, backend, enginefacade, engine, db_token):
+        self.backend = backend
+        self.enginefacade = enginefacade
+        self.engine = engine
+        self.db_token = db_token
+
+
+class Schema(object):
+    """"Represents a database schema that has or will be populated.
+
+    This is a marker object as required by testresources but otherwise
+    serves no purpose.
+
+    """
+    __slots__ = 'database',
 
 
 class BackendResource(testresources.TestResourceManager):
-    def __init__(self, database_type):
+    def __init__(self, database_type, ad_hoc_url=None):
         super(BackendResource, self).__init__()
         self.database_type = database_type
         self.backend = Backend.backend_for_database_type(self.database_type)
+        self.ad_hoc_url = ad_hoc_url
+        if ad_hoc_url is None:
+            self.backend = Backend.backend_for_database_type(
+                self.database_type)
+        else:
+            self.backend = Backend(self.database_type, ad_hoc_url)
+            self.backend._verify()
 
     def make(self, dependency_resources):
         return self.backend
 
+    def clean(self, resource):
+        self.backend._dispose()
+
     def isDirty(self):
         return False
 
 
 class DatabaseResource(testresources.TestResourceManager):
+    """Database resource which connects and disconnects to a URL.
 
-    def __init__(self, database_type):
+    For SQLite, this means the database is created implicitly, as a result
+    of SQLite's usual behavior.  If the database is a file-based URL,
+    it will remain after the resource has been torn down.
+
+    For all other kinds of databases, the resource indicates to connect
+    and disconnect from that database.
+
+    """
+
+    def __init__(self, database_type, _enginefacade=None,
+                 provision_new_database=True, ad_hoc_url=None):
         super(DatabaseResource, self).__init__()
         self.database_type = database_type
+        self.provision_new_database = provision_new_database
+
+        # NOTE(zzzeek) the _enginefacade is an optional argument
+        # here in order to accomodate Neutron's current direct use
+        # of the DatabaseResource object.  Within oslo_db's use,
+        # the "enginefacade" will always be passed in from the
+        # test and/or fixture.
+        if _enginefacade:
+            self._enginefacade = _enginefacade
+        else:
+            self._enginefacade = enginefacade._context_manager
         self.resources = [
-            ('backend', BackendResource(database_type))
+            ('backend', BackendResource(database_type, ad_hoc_url))
         ]
 
     def make(self, dependency_resources):
-        dependency_resources['db_token'] = db_token = _random_ident()
         backend = dependency_resources['backend']
-        LOG.info(
-            "CREATE BACKEND %s TOKEN %s", backend.engine.url, db_token)
-        backend.create_named_database(db_token, conditional=True)
-        dependency_resources['engine'] = \
-            backend.provisioned_engine(db_token)
-        return ProvisionedDatabase()
+        _enginefacade = self._enginefacade.make_new_manager()
+
+        if self.provision_new_database:
+            db_token = _random_ident()
+            url = backend.provisioned_database_url(db_token)
+            LOG.info(
+                "CREATE BACKEND %s TOKEN %s", backend.engine.url, db_token)
+            backend.create_named_database(db_token, conditional=True)
+        else:
+            db_token = None
+            url = backend.url
+
+        _enginefacade.configure(
+            logging_name="%s@%s" % (self.database_type, db_token))
+
+        _enginefacade._factory._start(connection=url)
+        engine = _enginefacade._factory._writer_engine
+        return ProvisionedDatabase(backend, _enginefacade, engine, db_token)
 
     def clean(self, resource):
-        resource.engine.dispose()
-        LOG.info(
-            "DROP BACKEND %s TOKEN %s",
-            resource.backend.engine, resource.db_token)
-        resource.backend.drop_named_database(resource.db_token)
+        if self.provision_new_database:
+            LOG.info(
+                "DROP BACKEND %s TOKEN %s",
+                resource.backend.engine, resource.db_token)
+            resource.backend.drop_named_database(resource.db_token)
 
     def isDirty(self):
         return False
 
 
+@debtcollector.removals.removed_class("TransactionResource")
 class TransactionResource(testresources.TestResourceManager):
 
     def __init__(self, database_resource, schema_resource):
@@ -105,10 +182,6 @@ class TransactionResource(testresources.TestResourceManager):
         return True
 
 
-class Schema(object):
-    pass
-
-
 class SchemaResource(testresources.TestResourceManager):
 
     def __init__(self, database_resource, generate_schema, teardown=False):
@@ -158,7 +231,6 @@ class Backend(object):
         self.engine = None
         self.impl = BackendImpl.impl(database_type)
         self.current_dbs = set()
-        Backend.backends_by_database_type[database_type] = self
 
     @classmethod
     def backend_for_database_type(cls, database_type):
@@ -168,7 +240,8 @@ class Backend(object):
         try:
             backend = cls.backends_by_database_type[database_type]
         except KeyError:
-            raise exception.BackendNotAvailable(database_type)
+            raise exception.BackendNotAvailable(
+                "Backend '%s' is unavailable: No such backend" % database_type)
         else:
             return backend._verify()
 
@@ -198,14 +271,15 @@ class Backend(object):
         if not self.verified:
             try:
                 eng = self._ensure_backend_available(self.url)
-            except exception.BackendNotAvailable:
+            except exception.BackendNotAvailable as bne:
+                self._no_engine_reason = str(bne)
                 raise
             else:
                 self.engine = eng
             finally:
                 self.verified = True
         if self.engine is None:
-            raise exception.BackendNotAvailable(self.database_type)
+            raise exception.BackendNotAvailable(self._no_engine_reason)
         return self
 
     @classmethod
@@ -220,7 +294,9 @@ class Backend(object):
             LOG.info(
                 _LI("The %(dbapi)s backend is unavailable: %(err)s"),
                 dict(dbapi=url.drivername, err=i_e))
-            raise exception.BackendNotAvailable("No DBAPI installed")
+            raise exception.BackendNotAvailable(
+                "Backend '%s' is unavailable: No DBAPI installed" %
+                url.drivername)
         else:
             try:
                 conn = eng.connect()
@@ -232,11 +308,17 @@ class Backend(object):
                     _LI("The %(dbapi)s backend is unavailable: %(err)s"),
                     dict(dbapi=url.drivername, err=d_e)
                 )
-                raise exception.BackendNotAvailable("Could not connect")
+                raise exception.BackendNotAvailable(
+                    "Backend '%s' is unavailable: Could not connect" %
+                    url.drivername)
             else:
                 conn.close()
                 return eng
 
+    def _dispose(self):
+        """Dispose main resources of this backend."""
+        self.impl.dispose(self.engine)
+
     def create_named_database(self, ident, conditional=False):
         """Create a database with the given name."""
 
@@ -267,6 +349,20 @@ class Backend(object):
 
         return self.impl.database_exists(self.engine, ident)
 
+    def provisioned_database_url(self, ident):
+        """Given the identifier of an anoymous database, return a URL.
+
+        For hostname-based URLs, this typically involves switching just the
+        'database' portion of the URL with the given name and creating
+        a URL.
+
+        For SQLite URLs, the identifier may be used to create a filename
+        or may be ignored in the case of a memory database.
+
+        """
+        return self.impl.provisioned_database_url(self.url, ident)
+
+    @debtcollector.removals.remove()
     def provisioned_engine(self, ident):
         """Given the URL of a particular database backend and the string
 
@@ -274,14 +370,6 @@ class Backend(object):
         an Engine instance whose connections will refer directly to the
         named database.
 
-        For hostname-based URLs, this typically involves switching just the
-        'database' portion of the URL with the given name and creating
-        an engine.
-
-        For URLs that instead deal with DSNs, the rules may be more custom;
-        for example, the engine may need to connect to the root URL and
-        then emit a command to switch to the named database.
-
         """
         return self.impl.provisioned_engine(self.url, ident)
 
@@ -312,8 +400,9 @@ class Backend(object):
         for url_str in configured_urls:
             url = sa_url.make_url(url_str)
             m = re.match(r'([^+]+?)(?:\+(.+))?$', url.drivername)
-            database_type, drivertype = m.group(1, 2)
-            Backend(database_type, url)
+            database_type = m.group(1)
+            Backend.backends_by_database_type[database_type] = \
+                Backend(database_type, url)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -331,6 +420,10 @@ class BackendImpl(object):
 
     supports_drop_fk = True
 
+    def dispose(self, engine):
+        LOG.info("DISPOSE ENGINE %s", engine)
+        engine.dispose()
+
     @classmethod
     def all_impls(cls):
         """Return an iterator of all possible BackendImpl objects.
@@ -364,7 +457,7 @@ class BackendImpl(object):
     def create_opportunistic_driver_url(self):
         """Produce a string url known as the 'opportunistic' URL.
 
-        This URL is one that corresponds to an established Openstack
+        This URL is one that corresponds to an established OpenStack
         convention for a pre-established database login, which, when
         detected as available in the local environment, is automatically
         used as a test platform for a specific type of driver.
@@ -422,6 +515,28 @@ class BackendImpl(object):
     def drop_additional_objects(self, conn):
         pass
 
+    def provisioned_database_url(self, base_url, ident):
+        """Return a provisioned database URL.
+
+        Given the URL of a particular database backend and the string
+        name of a particular 'database' within that backend, return
+        an URL which refers directly to the named database.
+
+        For hostname-based URLs, this typically involves switching just the
+        'database' portion of the URL with the given name and creating
+        an engine.
+
+        For URLs that instead deal with DSNs, the rules may be more custom;
+        for example, the engine may need to connect to the root URL and
+        then emit a command to switch to the named database.
+
+        """
+
+        url = sa_url.make_url(str(base_url))
+        url.database = ident
+        return url
+
+    @debtcollector.removals.remove()
     def provisioned_engine(self, base_url, ident):
         """Return a provisioned engine.
 
@@ -439,9 +554,8 @@ class BackendImpl(object):
         then emit a command to switch to the named database.
 
         """
+        url = self.provisioned_database_url(base_url, ident)
 
-        url = sa_url.make_url(str(base_url))
-        url.database = ident
         return session.create_engine(
             url,
             logging_name="%s@%s" % (self.drivername, ident),
@@ -452,10 +566,11 @@ class BackendImpl(object):
 @BackendImpl.impl.dispatch_for("mysql")
 class MySQLBackendImpl(BackendImpl):
 
+    # only used for deprecated provisioned_engine() function.
     default_engine_kwargs = {'mysql_sql_mode': 'TRADITIONAL'}
 
     def create_opportunistic_driver_url(self):
-        return "mysql://openstack_citest:openstack_citest@localhost/"
+        return "mysql+pymysql://openstack_citest:openstack_citest@localhost/"
 
     def create_named_database(self, engine, ident, conditional=False):
         with engine.connect() as conn:
@@ -476,22 +591,29 @@ class SQLiteBackendImpl(BackendImpl):
 
     supports_drop_fk = False
 
+    def dispose(self, engine):
+        LOG.info("DISPOSE ENGINE %s", engine)
+        engine.dispose()
+        url = engine.url
+        self._drop_url_file(url, True)
+
+    def _drop_url_file(self, url, conditional):
+        filename = url.database
+        if filename and (not conditional or os.access(filename, os.F_OK)):
+            os.remove(filename)
+
     def create_opportunistic_driver_url(self):
         return "sqlite://"
 
     def create_named_database(self, engine, ident, conditional=False):
-        url = self._provisioned_database_url(engine.url, ident)
+        url = self.provisioned_database_url(engine.url, ident)
         filename = url.database
         if filename and (not conditional or not os.access(filename, os.F_OK)):
             eng = sqlalchemy.create_engine(url)
             eng.connect().close()
 
-    def provisioned_engine(self, base_url, ident):
-        return session.create_engine(
-            self._provisioned_database_url(base_url, ident))
-
     def drop_named_database(self, engine, ident, conditional=False):
-        url = self._provisioned_database_url(engine.url, ident)
+        url = self.provisioned_database_url(engine.url, ident)
         filename = url.database
         if filename and (not conditional or os.access(filename, os.F_OK)):
             os.remove(filename)
@@ -501,7 +623,7 @@ class SQLiteBackendImpl(BackendImpl):
         filename = url.database
         return not filename or os.access(filename, os.F_OK)
 
-    def _provisioned_database_url(self, base_url, ident):
+    def provisioned_database_url(self, base_url, ident):
         if base_url.database:
             return sa_url.make_url("sqlite:////tmp/%s.db" % ident)
         else:
@@ -530,7 +652,7 @@ class PostgresqlBackendImpl(BackendImpl):
                 conn.execute("DROP DATABASE %s" % ident)
 
     def drop_additional_objects(self, conn):
-        enums = compat_utils.get_postgresql_enums(conn)
+        enums = [e['name'] for e in sqlalchemy.inspect(conn).get_enums()]
 
         for e in enums:
             conn.execute("DROP TYPE %s" % e)
