@@ -28,9 +28,10 @@ from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import mapper
 
 from oslo_db import exception
+from oslo_db.sqlalchemy.compat import utils as compat_utils
 from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import exc_filters
-from oslo_db.sqlalchemy import test_base
+from oslo_db.tests.sqlalchemy import base as test_base
 from oslo_db.tests import utils as test_utils
 
 _TABLE_NAME = '__tmp__test__tmp__'
@@ -306,7 +307,8 @@ class TestNonExistentConstraintMySQL(
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
         # error are not the same depending on its version…
         self.assertIsInstance(matched.inner_exception,
-                              sqlalchemy.exc.InternalError)
+                              (sqlalchemy.exc.InternalError,
+                               sqlalchemy.exc.OperationalError))
         if matched.table is not None:
             self.assertEqual("resource_foo", matched.table)
         if matched.constraint is not None:
@@ -376,7 +378,8 @@ class TestNonExistentTableMySQL(
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
         # error are not the same depending on its version…
         self.assertIsInstance(matched.inner_exception,
-                              sqlalchemy.exc.InternalError)
+                              (sqlalchemy.exc.InternalError,
+                               sqlalchemy.exc.OperationalError))
         self.assertEqual("foo", matched.table)
 
 
@@ -664,22 +667,36 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
                 # from the "with session.begin_nested()"
                 except exception.DBError as dbe_inner:
 
-                    # first "cause" is the failed SAVEPOINT rollback
-                    # from inside of flush(), when it fails
-                    self.assertTrue(
-                        isinstance(
-                            dbe_inner.cause,
-                            exception.DBError
+                    if not compat_utils.sqla_110:
+                        # first "cause" is the failed SAVEPOINT rollback
+                        # from inside of flush(), when it fails
+                        self.assertTrue(
+                            isinstance(
+                                dbe_inner.cause,
+                                exception.DBError
+                            )
                         )
-                    )
 
-                    # second "cause" is then the actual DB duplicate
-                    self.assertTrue(
-                        isinstance(
-                            dbe_inner.cause.cause,
-                            exception.DBDuplicateEntry
+                        # second "cause" is then the actual DB duplicate
+                        self.assertTrue(
+                            isinstance(
+                                dbe_inner.cause.cause,
+                                exception.DBDuplicateEntry
+                            )
                         )
-                    )
+                    else:
+                        # in SQLA 1.1, the rollback() method of Session
+                        # catches the error and repairs the state of the
+                        # session even though the SAVEPOINT was lost;
+                        # the net result here is that one exception is thrown
+                        # instead of two.  This is SQLAlchemy ticket #3680
+                        self.assertTrue(
+                            isinstance(
+                                dbe_inner.cause,
+                                exception.DBDuplicateEntry
+                            )
+                        )
+
         except exception.DBError as dbe_outer:
             self.assertTrue(
                 isinstance(
@@ -1097,7 +1114,7 @@ class IntegrationTest(test_base.DbTestCase):
         foo = self.Foo(counter=sqla.func.imfake(123))
         _session.add(foo)
         matched = self.assertRaises(sqla.exc.OperationalError, _session.flush)
-        self.assertTrue("no such function" in str(matched))
+        self.assertIn("no such function", str(matched))
 
     def test_query_wrapper_operational_error(self):
         """test an operational error from query.all() raised as-is."""
@@ -1109,7 +1126,7 @@ class IntegrationTest(test_base.DbTestCase):
         q = _session.query(self.Foo).filter(
             self.Foo.counter == sqla.func.imfake(123))
         matched = self.assertRaises(sqla.exc.OperationalError, q.all)
-        self.assertTrue("no such function" in str(matched))
+        self.assertIn("no such function", str(matched))
 
 
 class TestDBDisconnected(TestsExceptionFilter):
@@ -1162,6 +1179,14 @@ class TestDBDisconnected(TestsExceptionFilter):
         with self._fixture(dialect_name, exc_obj, 1):
             self.assertEqual(1, self.engine.scalar(sqla.select([1])))
 
+    def test_mariadb_error_1927(self):
+        for code in [1927]:
+            self._test_ping_listener_disconnected(
+                "mysql",
+                self.InternalError('%d Connection was killed' % code),
+                is_disconnect=False
+            )
+
     def test_mysql_ping_listener_disconnected(self):
         for code in [2006, 2013, 2014, 2045, 2055]:
             self._test_ping_listener_disconnected(
