@@ -42,8 +42,8 @@ from oslo_db.sqlalchemy.compat import utils as compat_utils
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import provision
 from oslo_db.sqlalchemy import session
-from oslo_db.sqlalchemy import test_base as db_test_base
 from oslo_db.sqlalchemy import utils
+from oslo_db.tests.sqlalchemy import base as db_test_base
 from oslo_db.tests import utils as test_utils
 
 
@@ -501,7 +501,7 @@ class TestMigrationUtils(db_test_base.DbTestCase):
 
         self.assertEqual(len(expected_ids), len(real_ids))
         for id_ in expected_ids:
-            self.assertTrue(id_ in real_ids)
+            self.assertIn(id_, real_ids)
 
     def test_drop_dup_entries_in_file_conn(self):
         table_name = "__test_tmp_table__"
@@ -541,7 +541,7 @@ class TestMigrationUtils(db_test_base.DbTestCase):
                    self.engine.execute(rows_select).fetchall()]
         self.assertEqual(len(expected_values), len(row_ids))
         for value in expected_values:
-            self.assertTrue(value['id'] in row_ids)
+            self.assertIn(value['id'], row_ids)
 
         deleted_rows_select = base_select.where(
             table.c.deleted == table.c.id)
@@ -551,7 +551,7 @@ class TestMigrationUtils(db_test_base.DbTestCase):
         self.assertEqual(len(values) - len(row_ids),
                          len(deleted_rows_ids))
         for value in soft_deleted_values:
-            self.assertTrue(value['id'] in deleted_rows_ids)
+            self.assertIn(value['id'], deleted_rows_ids)
 
     def test_change_deleted_column_type_does_not_drop_index(self):
         table_name = 'abc'
@@ -593,7 +593,7 @@ class TestMigrationUtils(db_test_base.DbTestCase):
         utils.change_deleted_column_type_to_id_type(self.engine, table_name)
 
         table = utils.get_table(self.engine, table_name)
-        self.assertTrue(isinstance(table.c.deleted.type, Integer))
+        self.assertIsInstance(table.c.deleted.type, Integer)
 
     def test_change_deleted_column_type_to_id_type_string(self):
         table_name = 'abc'
@@ -604,7 +604,7 @@ class TestMigrationUtils(db_test_base.DbTestCase):
         utils.change_deleted_column_type_to_id_type(self.engine, table_name)
 
         table = utils.get_table(self.engine, table_name)
-        self.assertTrue(isinstance(table.c.deleted.type, String))
+        self.assertIsInstance(table.c.deleted.type, String)
 
     @db_test_base.backend_specific('sqlite')
     def test_change_deleted_column_type_to_id_type_custom(self):
@@ -627,7 +627,7 @@ class TestMigrationUtils(db_test_base.DbTestCase):
 
         table = utils.get_table(self.engine, table_name)
 
-        self.assertTrue(isinstance(table.c.deleted.type, Integer))
+        self.assertIsInstance(table.c.deleted.type, Integer)
 
     def test_change_deleted_column_type_to_boolean(self):
         expected_types = {'mysql': mysql.TINYINT,
@@ -686,8 +686,8 @@ class TestMigrationUtils(db_test_base.DbTestCase):
         #                 but sqlalchemy will set it to NullType. This has
         #                 been fixed upstream in recent SA versions
         if SA_VERSION < (0, 9, 0):
-            self.assertTrue(isinstance(table.c.foo.type, NullType))
-        self.assertTrue(isinstance(table.c.deleted.type, Boolean))
+            self.assertIsInstance(table.c.foo.type, NullType)
+        self.assertIsInstance(table.c.deleted.type, Boolean)
 
     def test_detect_boolean_deleted_constraint_detection(self):
         table_name = 'abc'
@@ -1071,6 +1071,14 @@ class TestUtils(db_test_base.DbTestCase):
         self.test_table.create()
         self.addCleanup(meta.drop_all)
 
+    def test_get_indexes(self):
+        Index('index_a', self.test_table.c.a).create(self.engine)
+        Index('index_b', self.test_table.c.b).create(self.engine)
+        indexes = utils.get_indexes(self.engine, "test_table")
+        indexes = [(index['name'], index['column_names']) for index in indexes]
+        self.assertIn(('index_a', ['a']), indexes)
+        self.assertIn(('index_b', ['b']), indexes)
+
     def test_index_exists(self):
         self.assertFalse(utils.index_exists(self.engine, 'test_table',
                                             'new_index'))
@@ -1078,6 +1086,13 @@ class TestUtils(db_test_base.DbTestCase):
         self.assertTrue(utils.index_exists(self.engine, 'test_table',
                                            'new_index'))
 
+    def test_index_exists_on_columns(self):
+        columns = [self.test_table.c.a, self.test_table.c.b]
+        Index('new_index', *columns).create(self.engine)
+        self.assertTrue(utils.index_exists_on_columns(self.engine,
+                                                      'test_table',
+                                                      ('a', 'b')))
+
     def test_add_index(self):
         self.assertFalse(utils.index_exists(self.engine, 'test_table',
                                             'new_index'))
